==> glx.ui-1.8.21.tgz: package/components/rupViewer/rupViewer.ts <==
import { NgModule, Component, Input, OnInit, AfterViewInit, EventEmitter, Output } from '@angular/core';
import { AccordionModule } from 'primeng/components/accordion/accordion';
import { CommonModule } from '@angular/common';

import { DefaultOptions } from './defaultOptions';

declare let $: any;

@Component({
    selector: 'glx-rup-viewer',
    styleUrls: ['./rupViewer.scss'],
    templateUrl: './rupViewer.html'
})

export class RupViewer implements OnInit, AfterViewInit {
    @Input() data: any;
    @Input() multiple: boolean = true;
    @Input() allowSelection: boolean = false;
    @Input() options: any = DefaultOptions;

    electiveGroups: any = {};
    selection: boolean = false;
    error: string;
    _columns: any = [];

    @Output() onSave: EventEmitter<any> = new EventEmitter();

    constructor() {
    }

    ngOnInit() {
        this.collectElectives();
        this.initColumns();
    }

    initColumns() {
        // формируем массив колонок, которые нужно показывать
        for (let i = 0; i < this.options.columns.length; i++) {
            if (this.options.columns[i].show &&
                (this.options.columns[i].show === 1 || this.options.columns[i].show === true)) {
                this._columns.push(this.options.columns[i]);
            }
        }
        // сортируем массив колонок
        this._columns.sort(function (a, b) {
            return (a.sort > b.sort) ? 1 : ((b.sort > a.sort) ? -1 : 0);
        });
    }

    /*
    Переключение режима выбор дисциплин и просмотр
     */
    toggleSelection() {
        this.selection = !this.selection;
        this.error = '';
        this.calculateHoverEffects();
        if (this.selection) {
            this.collectElectives();
        } else {
            this.electiveGroups = {};
        }
    }

    /**
     * Собираем секции по выбору
     */
    collectElectives() {
        this.electiveGroups = {};
        if (this.data && this.data.semesters && this.data.semesters.length > 0) {
            for (let i = 0; i < this.data.semesters.length; i++) {
                for (let j = 0; j < this.data.semesters[i].sections.length; j++) {
                    const item = this.data.semesters[i].sections[j];
                    if (item.elective && item.elective === 1) {
                        this.electiveGroups[item.id] = null;
                    }
                    if (item.selected && (item.selected === 1 || item.selected === true)) {
                        this.electiveGroups[item.parent_id] = item.id;
                        if (this.allowSelection) {
                            this.allowSelection = false;
                        }
                    }
                }
            }
        }
    }

    /**
     * Проверяем, принадлежит ли предмет секции по выбору
     * @param parent_id ИД секции
     * @returns {boolean}
     */
    checkElective(parent_id) {
        return this.electiveGroups.hasOwnProperty(parent_id) && this.selection;
    }

    /**
     * Устанавливаем в секцию выбранный предмет
     * @param id
     * @param parent_id
     */
    setElective(id, parent_id) {
        if (this.checkElective(parent_id)) {
            if (this.electiveGroups[parent_id] === id) {
                this.electiveGroups[parent_id] = null;
            } else {
                this.electiveGroups[parent_id] = id;
            }
        }
    }

    /**
     * Проверка, что выбраны все дисциплины по выбору в каждом разделе
     * @returns {boolean}
     */
    validElectives() {
        for (const prop in this.electiveGroups) {
            if (this.electiveGroups.hasOwnProperty(prop)) {
                if (this.electiveGroups[prop] === null) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Отправка выбранных предметов
     */
    save() {
        if (this.validElectives()) {
            if (confirm(this.options.confirmMessage)) {
                this.error = '';
                this.selection = false;
                this.onSave.emit({selection: this.electiveGroups});
            }
        } else {
            this.error = this.options.errorMessage;
        }
    }

    ngAfterViewInit() {
        this.calculateHoverEffects();
    }

    calculateHoverEffects() {
        let table, level;
        if (this.selection) {
            for (let j = 0; j < this.data.semesters.length; j++) {
                table = $('#table_' + j)[0];
                if (table && table.rows) {
                    const tableRows = table.rows;
                    for (let i = 1; i < tableRows.length; i++) {
                        $(tableRows[i]).unbind('mouseenter mouseleave');
                    }
                }
            }
        } else {
            if (this.data && this.data.semesters && this.data.semesters.length > 0 && !this.selection) {
                for (let j = 0; j < this.data.semesters.length; j++) {
                    table = $('#table_' + j)[0];
                    if (table && table.rows) {
                        const tableRows = table.rows;
                        for (let i = 1; i < tableRows.length; i++) {
                            $(tableRows[i]).hover(
                                function () {
                                    level = tableRows[i].getAttribute('level');
                                    for (let ii = i + 1; ii < tableRows.length; ii++) {
                                        if (tableRows[ii].getAttribute('level') > level) {
                                            $(tableRows[ii]).addClass('hover');
                                        } else {
                                            break;
                                        }
                                    }
                                },
                                function () {
                                    for (let ii = 0; ii < tableRows.length; ii++) {
                                        $(tableRows[ii]).removeClass('hover');
                                    }
                                }
                            );
                        }
                    }
                }
            }
        }

    }

    getOffset(offset) {
        const base = 16;
        const padding = 24;
        if (!offset || offset === 0 || offset === '' || offset === '0') {
            return base;
        }

        return offset * padding + base;
    }

    getControlTypes(item) {
        let exam = 0;
        let test = 0;

        if (item && item.terms && item.terms[0] &&
            (item.terms[0].exam || item.terms[0].test)
        ) {
            exam = +item.terms[0].exam;
            test = +item.terms[0].test;
        } else {
            return '';
        }

        let examNum = '';
        let testNum = '';
        let result = '';

        if (exam > 1) {
            examNum = ' [' + exam + ']';
        }
        if (test > 1) {
            testNum = ' [' + test + ']';
        }

        if (exam > 0) {
            result = 'Экзамен' + examNum + ', ';
        }
        if (test > 0) {
            result += 'Зачет' + testNum + ', ';
        }
        return result.slice(0, result.length - 2);
    }

    emptyRup() {
        return !(this.data && this.data.semesters && this.data.semesters.length > 0);
    }
}

@NgModule({
    imports: [CommonModule, AccordionModule],
    exports: [RupViewer],
    declarations: [RupViewer]
})

export class RupViewerModule {
}

==> glx.ui-1.9.3.tgz: package/components/rupViewer/rupViewer.ts <==
import { NgModule, Component, Input, OnInit, AfterViewInit, EventEmitter, Output } from '@angular/core';
import { AccordionModule } from 'primeng/components/accordion/accordion';
import { CommonModule } from '@angular/common';

import { DefaultOptions } from './defaultOptions';

declare let $: any;

@Component({
    selector: 'glx-rup-viewer',
    styleUrls: ['./rupViewer.scss'],
    templateUrl: './rupViewer.html'
})

export class RupViewer implements OnInit, AfterViewInit {
    @Input() data: any;
    @Input() multiple: boolean = true;
    @Input() allowSelection: boolean = false;
    @Input() options: any = DefaultOptions;

    electiveGroups: any = {};
    selection: boolean = false;
    error: string;
    _columns: any = [];

    @Output() onSave: EventEmitter<any> = new EventEmitter();

    constructor() {
    }

    ngOnInit() {
        this.collectElectives();
        this.initColumns();
    }

    initColumns() {
        // формируем массив колонок, которые нужно показывать
        for (let i = 0; i < this.options.columns.length; i++) {
            if (this.options.columns[i].show &&
                (this.options.columns[i].show === 1 || this.options.columns[i].show === true)) {
                this._columns.push(this.options.columns[i]);
            }
        }
        // сортируем массив колонок
        this._columns.sort(function (a, b) {
            return (a.sort > b.sort) ? 1 : ((b.sort > a.sort) ? -1 : 0);
        });
    }

    /*
    Переключение режима выбор дисциплин и просмотр
     */
    toggleSelection() {
        this.selection = !this.selection;
        this.error = '';
        this.calculateHoverEffects();
        if (this.selection) {
            this.collectElectives();
        } else {
            this.electiveGroups = {};
        }
    }

    /**
     * Собираем секции по выбору
     */
    collectElectives() {
        this.electiveGroups = {};
        if (this.data && this.data.semesters && this.data.semesters.length > 0) {
            for (let i = 0; i < this.data.semesters.length; i++) {
                for (let j = 0; j < this.data.semesters[i].sections.length; j++) {
                    const item = this.data.semesters[i].sections[j];
                    if (item.sub_terms && item.sub_terms.length > 0 && item.terms && item.terms.length > 0) {
                        item.term_key = item.sub_terms.indexOf(this.data.semesters[i].semester);
                    } else {
                        item.term_key = null;
                    }
                    if (item.elective && item.elective === 1) {
                        this.electiveGroups[item.id] = null;
                    }
                    if (item.selected && (item.selected === 1 || item.selected === true)) {
                        this.electiveGroups[item.parent_id] = item.id;
                        if (this.allowSelection) {
                            this.allowSelection = false;
                        }
                    }
                }
            }
            console.log(this.data);
        }
    }

    /**
     * Проверяем, принадлежит ли предмет секции по выбору
     * @param parent_id ИД секции
     * @returns {boolean}
     */
    checkElective(parent_id) {
        return this.electiveGroups.hasOwnProperty(parent_id) && this.selection;
    }

    /**
     * Устанавливаем в секцию выбранный предмет
     * @param id
     * @param parent_id
     */
    setElective(id, parent_id) {
        if (this.checkElective(parent_id)) {
            if (this.electiveGroups[parent_id] === id) {
                this.electiveGroups[parent_id] = null;
            } else {
                this.electiveGroups[parent_id] = id;
            }
        }
    }

    /**
     * Проверка, что выбраны все дисциплины по выбору в каждом разделе
     * @returns {boolean}
     */
    validElectives() {
        for (const prop in this.electiveGroups) {
            if (this.electiveGroups.hasOwnProperty(prop)) {
                if (this.electiveGroups[prop] === null) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Отправка выбранных предметов
     */
    save() {
        if (this.validElectives()) {
            if (confirm(this.options.confirmMessage)) {
                this.error = '';
                this.selection = false;
                this.onSave.emit({selection: this.electiveGroups});
            }
        } else {
            this.error = this.options.errorMessage;
        }
    }

    ngAfterViewInit() {
        this.calculateHoverEffects();
    }

    calculateHoverEffects() {
        let table, level;
        if (this.selection) {
            for (let j = 0; j < this.data.semesters.length; j++) {
                table = $('#table_' + j)[0];
                if (table && table.rows) {
                    const tableRows = table.rows;
                    for (let i = 1; i < tableRows.length; i++) {
                        $(tableRows[i]).unbind('mouseenter mouseleave');
                    }
                }
            }
        } else {
            if (this.data && this.data.semesters && this.data.semesters.length > 0 && !this.selection) {
                for (let j = 0; j < this.data.semesters.length; j++) {
                    table = $('#table_' + j)[0];
                    if (table && table.rows) {
                        const tableRows = table.rows;
                        for (let i = 1; i < tableRows.length; i++) {
                            $(tableRows[i]).hover(
                                function () {
                                    level = tableRows[i].getAttribute('level');
                                    for (let ii = i + 1; ii < tableRows.length; ii++) {
                                        if (tableRows[ii].getAttribute('level') > level) {
                                            $(tableRows[ii]).addClass('hover');
                                        } else {
                                            break;
                                        }
                                    }
                                },
                                function () {
                                    for (let ii = 0; ii < tableRows.length; ii++) {
                                        $(tableRows[ii]).removeClass('hover');
                                    }
                                }
                            );
                        }
                    }
                }
            }
        }

    }

    getOffset(offset) {
        const base = 16;
        const padding = 24;
        if (!offset || offset === 0 || offset === '' || offset === '0') {
            return base;
        }

        return offset * padding + base;
    }

    getControlTypes(item) {
        let exam = 0;
        let test = 0;

        if (item && item.terms && item.terms[item.term_key] &&
            (item.terms[item.term_key].exam || item.terms[item.term_key].test)
        ) {
            exam = +item.terms[item.term_key].exam;
            test = +item.terms[item.term_key].test;
        } else {
            return '';
        }

        let examNum = '';
        let testNum = '';
        let result = '';

        if (exam > 1) {
            examNum = ' [' + exam + ']';
        }
        if (test > 1) {
            testNum = ' [' + test + ']';
        }

        if (exam > 0) {
            result = 'Экзамен' + examNum + ', ';
        }
        if (test > 0) {
            result += 'Зачет' + testNum + ', ';
        }
        return result.slice(0, result.length - 2);
    }

    emptyRup() {
        return !(this.data && this.data.semesters && this.data.semesters.length > 0);
    }

    getSemName(semester) {
        if (this.options.showSemName) {
            if (semester % 2) {
                return ' (Осенний)';
            } else {
                return ' (Весенний)';
            }
        }

        return '';
    }
}

@NgModule({
    imports: [CommonModule, AccordionModule],
    exports: [RupViewer],
    declarations: [RupViewer]
})

export class RupViewerModule {
}
